///////////////////////////////////////////////////////////////////////////////
// Filename:    UT2k3PyStats.uc
// Version:     1.0.2
// Author:      Jean-Pierre 'Page' van der Poel <pheniks@softhome.net>
//              Based on LocalStats by
//              Michiel 'El Muerte' Hendriks <elmuerte@drunksnipers.com>
// Purpose:     Log game stats in a method easily readable by Python (www.python.org)
//              This is the main class and co-ordinates logging from all "sub" classes
// Licence:     GPL
///////////////////////////////////////////////////////////////////////////////

class UT2k3PyStats extends GameStats Config(System);

const VERSION = "104";
const GAME = "UT2003";

var string logname;
var GameStats OldGameStats;
var UT2k3PySpec PySpec; //the spectator used to log in-game msg'es (and bots)
var UT2k3PyGameRules PyGameRules; //the GameRules class used to log pickups
var Config bool bLogHRMsges; // HR = "Human Readable", used when making
                             // a HR synopsis of the game
var Config bool bLogChats; // Chats between players.
                           // Changing this does not affect logging deathmessages
var Config bool bLogPickups; // Whether or not to log pickups.
                             // Enabling this causes large logs
var config string sLogDir;   // Path to place logs relative to the .u file

var int intHRKey;
var string strCurrHRKey;
static final operator(44) string += (out coerce string A, coerce string B) { A = A $ B; return A; }

function NewInit()
{
  Level.Game.bLoggingGame = true;
  Level.Game.bEnableStatLogging = true;
  log("[NOTICE] UT2k3PyStats: Starting UT2k3PyStats version "$VERSION);
  PySpec = Level.Spawn(class'UT2k3PyStats.UT2k3PySpec');
  PySpec.PyStat = self;
  AddGameRules();
  intHRKey = 0;
  if (Level.Game.GameStats != None) OldGameStats = Level.Game.GameStats;
  Level.Game.GameStats = Self;
    logname = "UT2k3PyStats_"$GetServerPort()$"_"$Level.Year$"_"$Level.Month$"_"$Level.Day$"_"$Level.Hour$"_"$Level.Minute$"_"$Level.Second$"";
	TempLog = spawn(class 'FileLog');
  	if (TempLog!=None)
	  {
		TempLog.OpenLog(logname);
  		log("[NOTICE] UT2k3PyStats: Output Game stats to: "$logname$".log");
	  }
  	else
	  {
		log("[ERROR] UT2k3PyStats: Could not spawn Temporary Stats log");
  		Destroy();
	  }
  log("[NOTICE] UT2k3PyStats: Jean-Pierre 'Page' van der Poel <page@valinor.za.net>");
  bShowBots = True;
}

function AddGameRules()
{
	PyGameRules = spawn(class'UT2k3PyStats.UT2k3PyGameRules');
	PyGameRules.PyStat = self;
	// we place our GameRules first so as not to affect
	// any others
	if ( Level.Game.GameRulesModifiers != None )
		PyGameRules.NextGameRules = Level.Game.GameRulesModifiers;
	Level.Game.GameRulesModifiers = PyGameRules;
}

function LogStatsInfo()
{
  Logf2("{'StatsInfo':{'Version':'"$VERSION$"','Game':'"$GAME$"','HRMsges':'"$bLogHRMsges$"','Chats':'"$bLogChats$"','Pickups':'"$bLogPickups$"'}}");
}

function Init()
{
  // don't do anything
}

function string TimeStamp()
{
  local string seconds;
  seconds = ""$Level.TimeSeconds;
  return seconds;
}

function string Header()
{
	return ""$TimeStamp();
}
function string HRKey()
{
	// TODO: More advanced Key generation
  intHRKey += 1;
  strCurrHRKey = Header()$" - "$intHRKey;
  return strCurrHRKey;
}
function string Difficulty(float Diff)
{
  if (Diff <= 0)
     return "Novice";
  if (Diff <= 1)
     return "Average";
  if (Diff <= 2)
     return "Experienced";
  if (Diff <= 3)
     return "Skilled";
  if (Diff <= 4)
     return "Adept";
  if (Diff <= 5)
     return "Masterful";
  if (Diff <= 6)
     return "Inhuman";
  return "Godlike";
}
function NewGame()
{
	local string out, tmp;
	local string ngTitle, ngAuthor, ngGameGameName;
	local bool NoGo;
	local mutator MyMutie;
	local GameRules MyRules;
	ngTitle			= Level.Title;			// Making local copies
	ngAuthor		= Level.Author;
	ngGameGameName	= Level.Game.GameName;
	GRI = Level.Game.GameReplicationInfo;
	out = "{'NewGame':{'TimeStamp':'"$Header()$"',";				// "NewGame"
	out += ("'FullTimeDate':'"$FullTimeDate()$"',");		// Game server's local time
	out += ("'TimeZone':'"$TimeZone()$"',");			// Game server's time zone (offset to GMT)
	out += ("'MapName':'"$MapName()$"',");				// Map file name without map extention .ut2
	out += ("'Title':'"$ngTitle$"',");
	out += ("'Author':'"$ngAuthor$"',");
	out += ("'GameClass':'"$Level.Game.Class$"',");
	out += ("'GameName':'"$ngGameGameName$"',");
	out += ("'Difficulty':'"$Difficulty(Level.Game.GameDifficulty)$"',");

    tmp = "";
    MyMutie = Level.Game.BaseMutator;
    if (MyMutie != None) {
       tmp="['"$MyMutie.Class$"'";
       MUTATORS:
         MyMutie = MyMutie.NextMutator;
         if (MyMutie != None) {
            tmp += (",'"$MyMutie.Class$"'");
            goto MUTATORS;
         }
    } else { tmp = "["; }
    out += ("'Mutators':"$tmp$"]");

    tmp = "";
    NoGo = false;
    MyRules = Level.Game.GameRulesModifiers;
    if (MyRules != None) {
       if (MyRules.IsA('UT2k3PyGameRules'))  //not logging our own GameRules almost makes spaghetti code :P
         NoGo = true;
       if (!NoGo)
           tmp = "'"$MyRules.Class$"'";
       GAMERULES:
         MyRules = MyRules.NextGameRules;
         if (MyRules != None) {
            if (!MyRules.IsA('UT2k3PyGameRules'))
               if (NoGo) {
                   tmp += ("'"$MyRules.Class$"'");
                   NoGo = false;
               } else
                   tmp += (",'"$MyRules.Class$"'");
            goto GAMERULES;
         }
    }
    out += (",'GameRules':["$tmp$"]");
    out += "}}";
	Logf2(out);
	LogStatsInfo();
	if (OldGameStats!=None) OldGameStats.NewGame();
}

// Return the server's port number.
function string GetServerPort()
{
    local string S;
    local int i;
    // Figure out the server's port.
    S = Level.GetAddressURL();
    i = InStr( S, ":" );
    assert(i>=0);
    return Mid(S,i+1);
}

function string Trim(string source)
{
  local int x;
  local string Temp;
  Temp = "";
  for (x = 0; x <= Len(source); x++)
    if (Mid(source,x,1) != " " && Mid(source,x,1) != chr(9))
      Temp = Temp$Mid(source,x,1);
  return Temp;
}

function Logf(string LogString)
{
  Logf2(LogString);
  //if (OldGameStats!=None) OldGameStats.Logf(LogString);
}

function Logf2(string LogString)
{
    if (TempLog!=None) TempLog.Logf(LogString);
}

// Connect Events get fired every time a player connects to a server
function ConnectEvent(PlayerReplicationInfo Who)
{
  local string out;
  if ( Who.bOnlySpectator )			// Spectators should never show up in stats!
	return;
  out = "{'Connect':{'TimeStamp':'"$Header()$"',";
  out += ("'PlayerNum':'"$Controller(Who.Owner).PlayerNum$"',");
  out += ("'PlayerName':'"$Who.PlayerName$"',");
  if (Who.bBot)
    out +=  "'StatsID':'BOT',";
  else
    out += ("'StatsID':'"$PlayerController(Controller(Who.Owner)).GetPlayerIDHash()$"',");
  out += ("'IsBot':'"$Who.bBot$"',");
  out += ("'IsAdmin':'"$Who.bAdmin$"',");
  out += ("'Ping':'"$Who.Ping$"',");
  if (Who.Team == none)
    out += "'TeamName':''";
  else
    out += ("'TeamName':'"$Who.Team.TeamName$"'");
// This *always* gives 'M', so it will be disabled until I can get another way
//  if (Who.bIsFemale)
//	  out += ",'Sex':'F',";
//  else
//      out += ",'Sex':'M'";

  out += "}}";
  Logf2(out);
  if (OldGameStats!=None) OldGameStats.ConnectEvent(Who);
}

function DisconnectEvent(PlayerReplicationInfo Who)
{
	local string out;
	if ( Who.bOnlySpectator )			// Spectators should never show up in stats!
		return;
	out = "{'Disconnect':{'TimeStamp':'"$Header()$"','PlayerNum':'"$Controller(Who.Owner).PlayerNum$"','PlayerName':'"$Who.PlayerName$"'}}";
	Logf2(out);
	if (OldGameStats!=None) OldGameStats.DisconnectEvent(Who);
}

function ServerInfo()
{
	local string out, flags;
	local string siServerName, siAdminName, siAdminEmail;
	local GameInfo.ServerResponseLine ServerState;
	local int i;
	siServerName	= GRI.ServerName;	// Making local copies
	siAdminName		= GRI.AdminName;
	siAdminEmail	= GRI.AdminEmail;
    out = "{'ServerInfo':{'TimeStamp':'"$Header()$"',";
	out += ("'ServerName':'"$siServerName$"',");
	out += ("'TimeZone':'"$TimeZone()$"',");
	out += ("'AdminName':'"$siAdminName$"',");
	out += ("'AdminEmail':'"$siAdminEmail$"',");
	flags = "";							// Server flag / key combos
	Level.Game.GetServerDetails( ServerState );
	flags = "'Flags':['"$ServerState.ServerInfo[0].Key$"="$ServerState.ServerInfo[0].Value$"',";
	for( i=1;i<ServerState.ServerInfo.Length;i++ )
		if (i == ServerState.ServerInfo.Length -1)
		{
		  flags += ("'"$ServerState.ServerInfo[i].Key$"="$ServerState.ServerInfo[i].Value$"'");
		} else {
			flags += ("'"$ServerState.ServerInfo[i].Key$"="$ServerState.ServerInfo[i].Value$"',");
		}
    flags += "]";
	out	+= flags;
	out += "}}";
	Logf2(out);
	if (OldGameStats!=None) OldGameStats.ServerInfo();
}

function StartGame()
{
	Logf2("{'StartGame':{'TimeStamp':'"$Header()$"'}}");
	if (OldGameStats!=None) OldGameStats.StartGame();
}

function EndGame(string Reason)
{
	local string out;
	out = "{'EndGame':{'TimeStamp':'"$Header()$"','Reason':'"$Reason$"'}}";
	Logf2(out);
	if (OldGameStats!=None) OldGameStats.EndGame(Reason);

	SaveConfig();
}

function ScoreEvent(PlayerReplicationInfo Who, float Points, string Desc)
{
	if ( Who.bOnlySpectator )			// Just to be totally safe Spectators sends nothing
		return;
	Logf2("{'Score':{'TimeStamp':'"$Header()$"','PlayerNum':'"$Controller(Who.Owner).PlayerNum$"','Points':'"$Points$"','Desc':'"$Desc$"'}}");
	if (OldGameStats!=None) OldGameStats.ScoreEvent(Who,Points,Desc);
}
function TeamScoreEvent(int Team, float Points, string Desc)
{
	Logf2("{'TeamScore':{'TimeStamp':'"$Header()$"','Team':'"$Team$"','Points':'"$Points$"','Desc':'"$Desc$"'}}");
	if (OldGameStats!=None) OldGameStats.TeamScoreEvent(Team,Points,Desc);
}

function KillEvent(string Killtype, PlayerReplicationInfo Killer, PlayerReplicationInfo Victim, class<DamageType> Damage)
{
	local string out;
	if (Killer == Victim)
		Killtype = "S";
    out = "{'Kill':{'TimeStamp':'"$Header()$"','KillType':'"$Killtype$"',";
	if (Killer!=None)
	{
		out += ("'KillerNum':'"$Controller(Killer.Owner).PlayerNum$"',");
		// KillerWeapon no longer used, using damagetype
		out += ("'DamageType':'"$GetItemName(string(Damage))$"',");
	}
	else
		out += ("'KillerNum':'-1','DamageType':'"$GetItemName(string(Damage))$"',");
	out += ("'KillerLocation':'"$Killer.PlayerZone.LocationName$"',");
	out += ("'VictimLocation':'"$Victim.PlayerZone.LocationName$"',");
	// VictimNumber and VictimWeapon
	out += ("'VictimNum':'"$Controller(Victim.Owner).PlayerNum$"',");
	out += ("'VictimWeapon':'"$GetItemName(string(Controller(Victim.Owner).GetLastWeapon()))$"'");
	if (bLogHRMsges)
		out += (",'HRKey':'"$strCurrHRKey$"'");
	// Type killers tracked as player event (redundant Typing, removed from kill line)
	if ( PlayerController(Victim.Owner)!= None && PlayerController(Victim.Owner).bIsTyping)
	{
		if ( PlayerController(Killer.Owner) != PlayerController(Victim.Owner) )
			SpecialEvent(Killer, "type_kill");						// Killer killed typing victim
	}
    out += "}}";
	Logf2(out);
	if (OldGameStats!=None) OldGameStats.KillEvent(Killtype,Killer,Victim,Damage);
}

// Special Events are everything else regarding the player
function SpecialEvent(PlayerReplicationInfo Who, string Desc)
{
	local string playernum;
	local string playerloc;
	if (Who != None)
	{
	  if ( Who.bOnlySpectator )		// Avoid spectator suicide on console "type_kill"
		return;
      playernum = ""$Controller(Who.Owner).PlayerNum;
	  playerloc = Who.PlayerZone.LocationName;
	}
	else
	{
	  playernum = "-1";
	  playerloc = "(nowhere)";
	}
    Logf2("{'SpecialPlayer':{'TimeStamp':'"$Header()$"','PlayerNum':'"$playernum$"','PlayerLocation':'"$playerloc$"','Desc':'"$Desc$"'}}");
    if (OldGameStats!=None) OldGameStats.SpecialEvent(Who,Desc);
}


// Special events regarding the game
function GameEvent(string GEvent, string Desc, PlayerReplicationInfo Who)
{
	local string PlayerNum,PlayerLoc, geDesc;
	if (Who != None)
	{
		if ( Who.bOnlySpectator )		// Specator could cause NameChange, TeamChange! No longer.
			return;

		PlayerNum = ""$Controller(Who.Owner).PlayerNum;
    if (Who.PlayerZone != None)
      PlayerLoc = Who.PlayerZone.LocationName;
    else
      PlayerLoc = "(nowhere)";
	}
	else
  {
		PlayerNum = "-1";
    PlayerLoc = "(nowhere)";
  }
	geDesc	= Desc;
  Logf2("{'SpecialGame':{'TimeStamp':'"$Header()$"','Event':'"$GEvent$"','PlayerNum':'"$PlayerNum$"','PlayerLocation':'"$PlayerLoc$"','Desc':'"$geDesc$"'}}");
  if (OldGameStats!=None) OldGameStats.GameEvent(GEvent,Desc,Who);
}

// has to be called before the game starts
event PreBeginPlay()
{
	Super.PreBeginPlay();
  	NewInit();
}

defaultproperties
{
     bLogHRMsges=True
     bLogChats=True
     bLogPickups=True
     sLogDir="../UserLogs/"
}
